//***************************************************************************************************************/
//*  Base Key Macro                                                                                                  */
//*     kbw - Key Base Width                                                                                    */
//*     kbd - Key Base Depth                                                                                    */
//*     kbh - Key Base Height                                                                                   */
//*     kfw - Key Face Width                                                                                    */
//*     kfd - Key Face Depth                                                                                    */
//*     kfh - Key Face Height                                                                                   */
//*     kbr - Key Base Radius                                                                                   */
//*     kfr - Key Face Radius                                                                                   */
//*     kfi - Key Face Image                                                                                  */
//*                                                                                                             */
//***************************************************************************************************************/
#macro Key(kbw,kbd,kbh,kfw,kfd,kfh,kbr,kfr,kfi)                         //*                                     */ 
  #local fb  = kbr;                                                     //* Corner Radius
  #local fw  = kfw/2;                                                   //* Half Face Width
  #local fh  = kfh;                                                     //* Face Height
  
  #local fa1 = degrees(atan(fh/fw));                                    //* First Angle Estimate
  #local fr  = (fh*fh+fw*fw)/(2*fh);                                    //* First Radius Estimate
  #local fa2 = asind(fw/fr);                                            //* Second Angle Estimate
  
  #local fa  = (fa1+fa2)/2;                                             //* Averaged Angle Estimate 
  #local fdx = fb*cosd(fa);
  #local fdy = fb*sind(fa);
  #local fdw = fw-fdx;
  #local fr  = fdw/sind(fa);
  #local fdh = fr*(1-cosd(fa));
  #local fh  = fdh+fdy; 
  
  #local fa  = fa*(kfh/fh);
  #local fdx = fb*cosd(fa);
  #local fdy = fb*sind(fa);
  #local fdw = fw-fdx;
  #local fr  = fdw/sind(fa);
  #local fdh = fr*(1-cosd(fa));
  #local fh  = fdh+fdy;
  #local fw  = fdw+fdx;
  
  #local ktr = fr;                                                      //*  Top Radius                         */
  #local ksa = degrees(atan((kbw-kfw)/(2*kbh)));                       //*  Key Side Angle                     */
  #local kta = fa;                                                    //*  Key Top  Angle                     */
  #local kfa = degrees(atan((kbd-kfd)/(1*kbh)));                                                    //*  Key Front Angle                    */
  #local kba = kfa;                                                    //*  Key Back  Angle                    */
                                                                        //*                                     */
  #local oKeyCorner =
    object {
      union {
        torus { kbr-kfr, kfr }
        cylinder { < 0, 0, 0>< 0,-kbh*1.1,0>,kbr }
        cylinder { < 0, 0, 0>< 0, kfr,0>,kbr-kfr }
        }
      }
 
  #local oKeySide =
    object {
      union {
        #local dz = (kfd/2-kbr);
        object {oKeyCorner translate +dz*z}
        object {oKeyCorner translate -dz*z}
        cylinder { <-(kbr-kfr), 0,-(kfd/2-kbr)><-(kbr-kfr), 0, (kfd/2-kbr)>,kfr }
        box { <-(kbr-kfr), 0,-(kfd/2-kbr)>< (kbr-kfr), kfr, (kfd/2-kbr)> }
        box { <-kbr,-kbh*1.1,-(kfd/2-kbr)>< kbr, 0, (kfd/2-kbr)> }
        }
      #local dx = tand(kta-ksa);
      Shear_Trans(x,y-dx*x,z)
      }
 
  #local oKeyBody =
    object {
      union {
        object {
          difference {
            cylinder {< 0, 0,-kfd/2>< 0, 0, kfd/2>,ktr+kbh*1.1 }
            cylinder {< 0, 0,-(kfd/2+0.0001)>< 0, 0, (kfd/2+0.0001)>,ktr}
            }
          }
        object {
          difference {
            cylinder {< 0, 0,-(kfd/2-kfr)>< 0, 0, (kfd/2-kfr)>,ktr }
            cylinder {< 0, 0,-(kfd/2-kfr+0.0001)>< 0, 0, (kfd/2-kfr+0.0001)>,ktr-kfr}
            }
          }
        object {
          union {
            torus { ktr, kfr rotate 90*x translate -(kfd/2-kfr)*z }
            torus { ktr, kfr rotate 90*x translate  (kfd/2-kfr)*z }
            }
          }
        }
      translate ktr*y
      clipped_by { plane { -x, 0 rotate ( kta-ksa )*z } }
      }
 
  #local oKeyHalf =
    object {
      union {
        object { oKeySide }
        object { oKeyBody }
        }
      translate -ktr*y
      rotate -kta*z
      translate  ktr*y
      clipped_by { plane { x, 0 } }
      }
 
  #local oKeyWhole =
    object {
      union {
        object {oKeyHalf}
        object { oKeyHalf rotate 180*y }
        }
      }
 
  #local dz = kbh*tand(kfa);
  #local dz = ((kbw-kfw)/2)/kbh;
  #local oKeyFront =
    object {
      oKeyWhole
      Shear_Trans(x,y+dz*z,z)
      clipped_by { plane {  z , 0 }}
      }
 
  #local dz = kbh*tand(kba);
  #local dz = ((kbw-kfw)/2)/kbh;
  #local oKeyBack =
    object {
      oKeyWhole
      Shear_Trans(x,y-dz*z,z)
      clipped_by { plane {  -z , 0 }}
      }
 
  #local oKeyPlain =
    object {
      union {
        object { oKeyFront }
        object { oKeyBack  }
        }
      }
      
  #declare oKeyFace =
    object {
      cylinder { < 0, ktr, -kfd/2>< 0, ktr,  kfd/2>,ktr-kfr-0.00001 open}
      rotate -90*x
      clipped_by{ box { <-(kfw/2-kbr), -(kfd/2-kbr), -ktr>< (kfw/2-kbr), (kfd/2-kbr), 0>}}
      pigment{image_map{gif concat("MTXKeyboard/",kfi) transmit 13, 1} translate < -0.5, 0.5, 0> scale <kfw-2*kbr, kfd-2*kbr, 1> }
      rotate  90*x 
      } 
  
  #declare oKey =
    object {
      union {
        object { oKeyPlain }
        object { oKeyFace  }
        }
      translate kbh*y
      clipped_by{ plane { -y, 0}}  
      }
  
  oKey
 
#end


//***************************************************************************************************************/
//*  1.00 x 1.00 Key Macro                                                                                                  */
//***************************************************************************************************************/
#macro Key1x1(sKeyFaceImage)
  #local fkbw = fMTXKeyBaseWidth ;                                         //*                                     */
  #local fkbd = fMTXKeyBaseDepth ;                                         //*                                     */
  #local fkbh = fMTXKeyBaseHeight;                                         //*                                     */
  #local fkfw = fMTXKeyFaceWidth ;                                         //*                                     */
  #local fkfd = fMTXKeyFaceDepth ;                                         //*                                     */
  #local fkfh = fMTXKeyFaceHeight;                                         //*                                     */
  #local fkbr = fMTXKeyBaseRadius;                                         //*                                     */
  #local fkfr = fMTXKeyFaceRadius;                                         //*                                     */
  
  #local oKey1x1 =
    object {
      Key(fkbw,fkbd,fkbh,fkfw,fkfd,fkfh,fkbr,fkfr,sKeyFaceImage)       //*                                     */
      translate < fkbw/2, 0,-fkbd/2>
      }
  
  oKey1x1
  
#end    
      
//***************************************************************************************************************/
//*  1.20 x 1.00 Key Macro                                                                                                  */
//***************************************************************************************************************/
#macro Key120x1(sKeyFaceImage)
  #local fkbw = fMTXKeyBaseWidth+fMTXKeyWidth*0.2;                                         //*                                     */
  #local fkbd = fMTXKeyBaseDepth ;                                         //*                                     */
  #local fkbh = fMTXKeyBaseHeight;                                         //*                                     */
  #local fkfw = fMTXKeyFaceWidth+fMTXKeyWidth*0.2;                                         //*                                     */
  #local fkfd = fMTXKeyFaceDepth ;                                         //*                                     */
  #local fkfh = fMTXKeyFaceHeight;                                         //*                                     */
  #local fkbr = fMTXKeyBaseRadius;                                         //*                                     */
  #local fkfr = fMTXKeyFaceRadius;                                         //*                                     */
  
  #local oKey120x1 =
    object {
      Key(fkbw,fkbd,fkbh,fkfw,fkfd,fkfh,fkbr,fkfr,sKeyFaceImage)       //*                                     */
      translate < fkbw/2, 0,-fkbd/2>
      }
  
  oKey120x1
  
#end    
      
//***************************************************************************************************************/
//*  1.50 x 1.00 Key Macro                                                                                                  */
//***************************************************************************************************************/
#macro Key150x1(sKeyFaceImage)
  #local fkbw = fMTXKeyBaseWidth+fMTXKeyWidth*0.5;                                         //*                                     */
  #local fkbd = fMTXKeyBaseDepth ;                                         //*                                     */
  #local fkbh = fMTXKeyBaseHeight;                                         //*                                     */
  #local fkfw = fMTXKeyFaceWidth+fMTXKeyWidth*0.5;                                         //*                                     */
  #local fkfd = fMTXKeyFaceDepth ;                                         //*                                     */
  #local fkfh = fMTXKeyFaceHeight;                                         //*                                     */
  #local fkbr = fMTXKeyBaseRadius;                                         //*                                     */
  #local fkfr = fMTXKeyFaceRadius;                                         //*                                     */
  
  #local oKey150x1 =
    object {
      Key(fkbw,fkbd,fkbh,fkfw,fkfd,fkfh,fkbr,fkfr,sKeyFaceImage)       //*                                     */
      translate < fkbw/2, 0,-fkbd/2>
      }
  
  oKey150x1
  
#end    
      
//***************************************************************************************************************/
//*  1.75 x 1.00 Key Macro                                                                                                  */
//***************************************************************************************************************/
#macro Key175x1(sKeyFaceImage)
  #local fkbw = fMTXKeyBaseWidth+fMTXKeyWidth*0.75;                                         //*                                     */
  #local fkbd = fMTXKeyBaseDepth ;                                         //*                                     */
  #local fkbh = fMTXKeyBaseHeight;                                         //*                                     */
  #local fkfw = fMTXKeyFaceWidth+fMTXKeyWidth*0.75;                                         //*                                     */
  #local fkfd = fMTXKeyFaceDepth ;                                         //*                                     */
  #local fkfh = fMTXKeyFaceHeight;                                         //*                                     */
  #local fkbr = fMTXKeyBaseRadius;                                         //*                                     */
  #local fkfr = fMTXKeyFaceRadius;                                         //*                                     */
  
  #local oKey175x1 =
    object {
      Key(fkbw,fkbd,fkbh,fkfw,fkfd,fkfh,fkbr,fkfr,sKeyFaceImage)       //*                                     */
      translate < fkbw/2, 0,-fkbd/2>
      }
  
  oKey175x1
  
#end    
      
//***************************************************************************************************************/
//*  2.00 x 1.00 Key Macro                                                                                                  */
//***************************************************************************************************************/
#macro Key200x1(sKeyFaceImage)
  #local fkbw = fMTXKeyBaseWidth+fMTXKeyWidth*1.0;                                         //*                                     */
  #local fkbd = fMTXKeyBaseDepth ;                                         //*                                     */
  #local fkbh = fMTXKeyBaseHeight;                                         //*                                     */
  #local fkfw = fMTXKeyFaceWidth+fMTXKeyWidth*1.0;                                         //*                                     */
  #local fkfd = fMTXKeyFaceDepth ;                                         //*                                     */
  #local fkfh = fMTXKeyFaceHeight;                                         //*                                     */
  #local fkbr = fMTXKeyBaseRadius;                                         //*                                     */
  #local fkfr = fMTXKeyFaceRadius;                                         //*                                     */
  
  #local oKey200x1 =
    object {
      Key(fkbw,fkbd,fkbh,fkfw,fkfd,fkfh,fkbr,fkfr,sKeyFaceImage)       //*                                     */
      translate < fkbw/2, 0,-fkbd/2>
      }
  
  oKey200x1
  
#end    
      
//***************************************************************************************************************/
//*  2.25 x 1.00 Key Macro                                                                                                  */
//***************************************************************************************************************/
#macro Key225x1(sKeyFaceImage)
  #local fkbw = fMTXKeyBaseWidth+fMTXKeyWidth*1.25;                                         //*                                     */
  #local fkbd = fMTXKeyBaseDepth ;                                         //*                                     */
  #local fkbh = fMTXKeyBaseHeight;                                         //*                                     */
  #local fkfw = fMTXKeyFaceWidth+fMTXKeyWidth*1.25;                                         //*                                     */
  #local fkfd = fMTXKeyFaceDepth ;                                         //*                                     */
  #local fkfh = fMTXKeyFaceHeight;                                         //*                                     */
  #local fkbr = fMTXKeyBaseRadius;                                         //*                                     */
  #local fkfr = fMTXKeyFaceRadius;                                         //*                                     */
  
  #local oKey225x1 =
    object {
      Key(fkbw,fkbd,fkbh,fkfw,fkfd,fkfh,fkbr,fkfr,sKeyFaceImage)       //*                                     */
      translate < fkbw/2, 0,-fkbd/2>
      }
  
  oKey225x1
  
#end    
      
//***************************************************************************************************************/
//*  2.75 x 1.00 Key Macro                                                                                                  */
//***************************************************************************************************************/
#macro Key275x1(sKeyFaceImage)
  #local fkbw = fMTXKeyBaseWidth+fMTXKeyWidth*1.75;                                         //*                                     */
  #local fkbd = fMTXKeyBaseDepth ;                                         //*                                     */
  #local fkbh = fMTXKeyBaseHeight;                                         //*                                     */
  #local fkfw = fMTXKeyFaceWidth+fMTXKeyWidth*1.75;                                         //*                                     */
  #local fkfd = fMTXKeyFaceDepth ;                                         //*                                     */
  #local fkfh = fMTXKeyFaceHeight;                                         //*                                     */
  #local fkbr = fMTXKeyBaseRadius;                                         //*                                     */
  #local fkfr = fMTXKeyFaceRadius;                                         //*                                     */
  
  #local oKey275x1 =
    object {
      Key(fkbw,fkbd,fkbh,fkfw,fkfd,fkfh,fkbr,fkfr,sKeyFaceImage)       //*                                     */
      translate < fkbw/2, 0,-fkbd/2>
      }
  
  oKey275x1
  
#end    
      
//***************************************************************************************************************/
//*  6 x 1.00 Key Macro                                                                                                  */
//***************************************************************************************************************/
#macro Key6x1(sKeyFaceImage)
  #local fkbw = fMTXKeyBaseWidth+fMTXKeyWidth*5.00;                                         //*                                     */
  #local fkbd = fMTXKeyBaseDepth ;                                         //*                                     */
  #local fkbh = fMTXKeyBaseHeight;                                         //*                                     */
  #local fkfw = fMTXKeyFaceWidth+fMTXKeyWidth*5.00;                                         //*                                     */
  #local fkfd = fMTXKeyFaceDepth ;                                         //*                                     */
  #local fkfh = fMTXKeyFaceHeight;                                         //*                                     */
  #local fkbr = fMTXKeyBaseRadius;                                         //*                                     */
  #local fkfr = fMTXKeyFaceRadius;                                         //*                                     */
  
  #local oKey6x1 =
    object {
      Key(fkbw,fkbd,fkbh,fkfw,fkfd,fkfh,fkbr,fkfr,sKeyFaceImage)       //*                                     */
      translate < fkbw/2, 0,-fkbd/2>
      }
  
  oKey6x1
  
#end    
      
//***************************************************************************************************************/
//*  2.00 x 1.00 Key Macro                                                                                                  */
//***************************************************************************************************************/
#macro Key1x2(sKeyFaceImage)
  #local fkbw = fMTXKeyBaseWidth;                                         //*                                     */
  #local fkbd = fMTXKeyBaseDepth+fMTXKeyDepth*1.0 ;                                         //*                                     */
  #local fkbh = fMTXKeyBaseHeight;                                         //*                                     */
  #local fkfw = fMTXKeyFaceWidth;                                         //*                                     */
  #local fkfd = fMTXKeyFaceDepth+fMTXKeyDepth*1.0 ;                                         //*                                     */
  #local fkfh = fMTXKeyFaceHeight;                                         //*                                     */
  #local fkbr = fMTXKeyBaseRadius;                                         //*                                     */
  #local fkfr = fMTXKeyFaceRadius;                                         //*                                     */
  
  #local oKey1x2 =
    object {
      Key(fkbw,fkbd,fkbh,fkfw,fkfd,fkfh,fkbr,fkfr,sKeyFaceImage)       //*                                     */
      translate < fkbw/2, 0,-fkbd/2>
      }
  
  oKey1x2
  
#end    
      
//***************************************************************************************************************/
//*  Key Size Definitions                                                                                                */
//***************************************************************************************************************/
#declare fMTXKeyBaseWidth  = 0.70;                                         //*                                     */
#declare fMTXKeyBaseDepth  = 0.70;                                         //*                                     */
#declare fMTXKeyBaseHeight = 0.35;                                         //*                                     */
#declare fMTXKeyFaceWidth  = 0.50;                                         //*                                     */
#declare fMTXKeyFaceDepth  = 0.50;                                         //*                                     */
#declare fMTXKeyFaceHeight = 0.05;                                         //*                                     */
#declare fMTXKeyBaseRadius = 0.06;                                         //*                                     */
#declare fMTXKeyFaceRadius = 0.01;                                         //*                                     */
#declare fMTXKeyWidth      = 0.75;
#declare fMTXKeyDepth      = 0.75;
//#declare vMTXDarkKeyTexture  = pigment{White*0.50};
#declare vMTXDarkKeyTexture  = texture {
                        pigment{rgb <0.65,0.65,0.45>}                   //* Base Texture                        */
                        normal { bumps 0.04 scale 0.0005 }
                        finish { phong .5 }
                        };
//#declare vMTXLightKeyTexture = pigment{White*0.75};
#declare vMTXLightKeyTexture = texture {
                        pigment{rgb <0.85,0.85,0.65>}                   //* Base Texture                        */
                        normal { bumps 0.04 scale 0.0005 }
                        finish { phong .5 }
                        };
//***************************************************************************************************************/
//*  PF Keyboard Row 1                                                                                                 */
//***************************************************************************************************************/
#declare oPFKeyRow1 =
  object {
    union {  
      object {Key1x1("f1")                                        texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f2")            translate  1*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f3")            translate  2*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f4")            translate  3*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f5")            translate  4*fMTXKeyWidth*x texture{vMTXLightKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f6")            translate  5*fMTXKeyWidth*x texture{vMTXLightKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f7")            translate  6*fMTXKeyWidth*x texture{vMTXLightKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f8")            translate  7*fMTXKeyWidth*x texture{vMTXLightKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f9")            translate  8*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f10")           translate  9*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f11")           translate 10*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f12")           translate 11*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      }
    }
  
//***************************************************************************************************************/
//*  PF Keyboard Row 2                                                                                                */
//***************************************************************************************************************/
#declare oPFKeyRow2 =
  object {
    union {  
      object {Key1x1("f13")                                        texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f14")            translate  1*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f15")            translate  2*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f16")            translate  3*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f17")            translate  4*fMTXKeyWidth*x texture{vMTXLightKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f18")            translate  5*fMTXKeyWidth*x texture{vMTXLightKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f19")            translate  6*fMTXKeyWidth*x texture{vMTXLightKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f20")            translate  7*fMTXKeyWidth*x texture{vMTXLightKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f21")            translate  8*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f22")            translate  9*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f23")            translate 10*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("f24")            translate 11*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      }
    }
  
//***************************************************************************************************************/
//*  PF Keys                                                                                                */
//***************************************************************************************************************/
#declare PFKeys =  
  object {
    union {
      object {oPFKeyRow1 translate 1*fMTXKeyDepth*z}
      object {oPFKeyRow2 translate 2*fMTXKeyDepth*z}
      }
//  pigment {rgb <0.25,0.25,0.25>}  
    }
    
//***************************************************************************************************************/
//*  Keyboard Hole 1 Row 1 -5                                                                                               */
//***************************************************************************************************************/
#declare oKey1Row1 =
  object {
    union {  
      object {Key1x1("blank")                                      texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("blank")          translate  1*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      }
    }
  
#declare oKey1Row2 =
  object {
    union {  
      object {Key1x1("print")                                      texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("copy")           translate  1*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      }
    }
  
#declare oKey1Row3 =
  object {
    union {  
      object {Key1x1("exsel")                                      texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("ereof")          translate  1*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      }
    }
  
#declare oKey1Row4 =
  object {
    union {  
      object {Key1x1("crsel")                                      texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("pause")          translate  1*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      }
    }
  
#declare oKey1Row5 =
  object {
    union {  
      object {Key1x1("attn")                                       texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      object {Key1x1("clear")          translate  1*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      }
    }
  
#declare oKeys1 =  
  object {
    union {
      object {oKey1Row1 translate 1*fMTXKeyDepth*z}
      object {oKey1Row2 translate 2*fMTXKeyDepth*z}
      object {oKey1Row3 translate 3*fMTXKeyDepth*z}
      object {oKey1Row4 translate 4*fMTXKeyDepth*z}
      object {oKey1Row5 translate 5*fMTXKeyDepth*z}
      }
    }
    
//***************************************************************************************************************/
//*  Keyboard Row 1                                                                                                 */
//***************************************************************************************************************/
#declare oKeyRow1 =
  object {
    union {  
      object {Key1x1("tilde")                                }                                                                     //*                                     */
      object {Key1x1("1")            translate  1*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("2")            translate  2*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("3")            translate  3*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("4")            translate  4*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("5")            translate  5*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("6")            translate  6*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("7")            translate  7*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("8")            translate  8*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("9")            translate  9*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("0")            translate 10*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("minus")        translate 11*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key1x1("equal")        translate 12*fMTXKeyWidth*x}                                                                     //*                                     */
      object {Key200x1("backspace")  translate 13*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
      }
    }
  
//***************************************************************************************************************/
//*  Keyboard Row 2                                                                                                 */
//***************************************************************************************************************/
#declare oKeyRow2 =
  object {
    union {  
      object {Key150x1("tab")                                        texture{vMTXDarkKeyTexture}}                                                                    //*                                     */
      object {
        union {
          object {Key1x1("Q")            translate  0*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("W")            translate  1*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("E")            translate  2*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("R")            translate  3*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("T")            translate  4*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("Y")            translate  5*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("U")            translate  6*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("I")            translate  7*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("O")            translate  8*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("P")            translate  9*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("left_brace")   translate 10*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("right_brace")  translate 11*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key150x1("back_slash") translate 12*fMTXKeyWidth*x}                                                                     //*                                     */
          }
        translate 1.5*fMTXKeyWidth*x
        }
      }
    }
  
//***************************************************************************************************************/
//*  Keyboard Row 3                                                                                                */
//***************************************************************************************************************/
#declare oKeyRow3 =
  object {
    union {  
      object {Key175x1("caps_lock")                                  texture{vMTXDarkKeyTexture}}
      object {
        union {
          object {Key1x1("A")            translate  0*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("S")            translate  1*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("D")            translate  2*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("F")            translate  3*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("G")            translate  4*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("H")            translate  5*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("J")            translate  6*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("K")            translate  7*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("L")            translate  8*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("colon")        translate  9*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("quote")        translate 10*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key225x1("enter")      translate 11*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
          }
        translate 1.75*fMTXKeyWidth*x
        }
      }  
    }
  
//***************************************************************************************************************/
//*  Keyboard Row 4                                                                                                 */
//***************************************************************************************************************/
#declare oKeyRow4 =
  object {
    union {  
      object {Key225x1("shift_left")                                  texture{vMTXDarkKeyTexture}}
      object {
        union {                                                                                               //*                                     */
          object {Key1x1("Z")             translate  0*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("X")             translate  1*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("C")             translate  2*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("V")             translate  3*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("B")             translate  4*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("N")             translate  5*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("M")             translate  6*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("comma")         translate  7*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("period")        translate  8*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key1x1("question")      translate  9*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key275x1("shift_right") translate 10*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}
          }
        translate 2.25*fMTXKeyWidth*x
        }                                                                       //*                                     */
      }
    }
  
//***************************************************************************************************************/
//*  Keyboard Row 5                                                                                                */
//***************************************************************************************************************/
#declare oKeyRow5 =
  object {
    union {  
      object {Key150x1("ctrl")                                              texture{vMTXDarkKeyTexture}}
      object {
        union {                                                                                               //*                                     */
          object {Key120x1("left_arrow") translate  0       *fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
          object {Key120x1("alt")        translate  1*1.2   *fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
          object {Key6x1("space")        translate  2*1.2   *fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key120x1("alt")        translate (2*1.2+6)*fMTXKeyWidth*x}                                                                     //*                                     */
          object {Key120x1("left_arrow") translate (3*1.2+6)*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
          object {Key120x1("left_arrow") translate (4*1.2+6)*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
          object {Key150x1("ctrl")       translate (5*1.2+6)*fMTXKeyWidth*x texture{vMTXDarkKeyTexture}}                                                                     //*                                     */
          }
        translate 1.50*fMTXKeyWidth*x
        }                                                                       //*                                     */
      }
    }

#declare Keys =  
  object {
    union {
      object {oKeyRow1 translate 5*fMTXKeyDepth*z}
      object {oKeyRow2 translate 4*fMTXKeyDepth*z}
      object {oKeyRow3 translate 3*fMTXKeyDepth*z}
      object {oKeyRow4 translate 2*fMTXKeyDepth*z}
      object {oKeyRow5 translate 1*fMTXKeyDepth*z}
      }
    texture{vMTXLightKeyTexture}    
    }
    
#declare Keys3 =
  object {
    union {
      object { Key1x1("page_down")   translate < 2*fMTXKeyWidth,0, 5*fMTXKeyDepth>}
      object { Key1x1("page_up")     translate < 1*fMTXKeyWidth,0, 5*fMTXKeyDepth>}
      object { Key1x1("blank")       translate < 0*fMTXKeyWidth,0, 5*fMTXKeyDepth>}
      object { Key1x1("delete")      translate < 2*fMTXKeyWidth,0, 4*fMTXKeyDepth>}
      object { Key1x1("insert")      translate < 1*fMTXKeyWidth,0, 4*fMTXKeyDepth>}
      object { Key1x1("end")         translate < 0*fMTXKeyWidth,0, 4*fMTXKeyDepth>}
      object { Key1x1("up_arrow")    translate < 1*fMTXKeyWidth,0, 3*fMTXKeyDepth>}
      object { Key1x1("left_arrow")  translate < 0*fMTXKeyWidth,0, 2*fMTXKeyDepth>}
      object { Key1x1("rule")        translate < 1*fMTXKeyWidth,0, 2*fMTXKeyDepth>}
      object { Key1x1("right_arrow") translate < 2*fMTXKeyWidth,0, 2*fMTXKeyDepth>}
      object { Key1x1("down_arrow")  translate < 1*fMTXKeyWidth,0, 1*fMTXKeyDepth>}
      }
    texture{vMTXDarkKeyTexture}
    }
    
#declare Keys4 =
  object {
    union {
      object { Key1x1("numlock")       translate <              0,0, 5*fMTXKeyDepth> texture{vMTXDarkKeyTexture}}
      object { Key1x1("slash")         translate <   fMTXKeyWidth,0, 5*fMTXKeyDepth> texture{vMTXDarkKeyTexture}}
      object { Key1x1("star")          translate < 2*fMTXKeyWidth,0, 5*fMTXKeyDepth> texture{vMTXDarkKeyTexture}}
      object { Key1x1("minus")         translate < 3*fMTXKeyWidth,0, 5*fMTXKeyDepth> texture{vMTXDarkKeyTexture}}
      object { Key1x1("7home")         translate <              0,0, 4*fMTXKeyDepth>}
      object { Key1x1("8up")           translate <   fMTXKeyWidth,0, 4*fMTXKeyDepth>}
      object { Key1x1("9pgup")         translate < 2*fMTXKeyWidth,0, 4*fMTXKeyDepth>}
      object { Key1x2("plus")          translate < 3*fMTXKeyWidth,0, 4*fMTXKeyDepth> texture{vMTXDarkKeyTexture}}
      object { Key1x1("4left")         translate <              0,0, 3*fMTXKeyDepth>}
      object { Key1x1("5")             translate <   fMTXKeyWidth,0, 3*fMTXKeyDepth>}
      object { Key1x1("6right")        translate < 2*fMTXKeyWidth,0, 3*fMTXKeyDepth>}
      object { Key1x1("1end")          translate <              0,0, 2*fMTXKeyDepth>}
      object { Key1x1("2down")         translate <   fMTXKeyWidth,0, 2*fMTXKeyDepth>}
      object { Key1x1("3pgdn")         translate < 2*fMTXKeyWidth,0, 2*fMTXKeyDepth>}
      object { Key1x2("entertall")     translate < 3*fMTXKeyWidth,0, 2*fMTXKeyDepth> texture{vMTXDarkKeyTexture}}
      object { Key200x1("0ins")        translate <              0,0, 1*fMTXKeyDepth>}
      object { Key1x1("del")           translate < 2*fMTXKeyWidth,0, 1*fMTXKeyDepth>}
      }
    texture{vMTXLightKeyTexture}
    }

                                                                        //*                                     */
//***************************************************************************************************************/
 

